c USGS_GRADCOMP
c
c computes horizontal derivatives and horizontal gradient magnitude using
c the gradient-component method of Thurston and Brown (Geophysics, 1994).
c
c source code modified from Thurston and Brown (1994) by Jeff Phillips.
c
      subroutine gradcomp(infile,outfilh,outfilx,outfily,nobs,norder,
     1ierr)
c
      parameter(maxnobs=21, ncmax=8000)
c
      character*(*) infile,outfilh,outfilx,outfily
      integer*4 inunit, outhunit, outxunit, outyunit
      integer*4 i,j,k,mid,nobs
      real filtop1(maxnobs*maxnobs), filtop2(maxnobs*maxnobs)
      real surf(ncmax,maxnobs),grad(ncmax),xx(ncmax),yy(ncmax)
      data inunit/0/,outhunit/0/,outxunit/0/,outyunit/0/
      data dummy/-1.0e32/
c
      call OpenGrid_WF(infile,inunit,ierr)
      if(ierr.ne.0) go to 9999
      call GetGridInfo_WF(inunit,nc,nr,xo,yo,dx,dy,rot,ierr)
      if(ierr.ne.0) go to 9999
      if(nc.gt.ncmax) then
        call RegisterErr_WF(2002,'usgs')
        call SetErrParmI_WF(1,nc)
        call SetErrParmI_WF(2,ncmax)
        ierr=1
        go to 9999
      endif
      call NewGrid_WF(outhunit,outfilh,nc,nr,xo,yo,dx,dy,rot,ierr)
      if(ierr.ne.0) go to 9999
      call NewGrid_WF(outxunit,outfilx,nc,nr,xo,yo,dx,dy,rot,ierr)
      if(ierr.ne.0) go to 9999
      call NewGrid_WF(outyunit,outfily,nc,nr,xo,yo,dx,dy,rot,ierr)
      if(ierr.ne.0) go to 9999
c
c  DEBUGING CODE
c       call RegisterErr_WF(2000,'usgs')
c       call SetErrParmI_WF(1,nobs)
c       call SetErrParmI_WF(2,norder)
c       ierr=1
c       go to 9999
c
      call gradop(filtop1,filtop2,nobs,norder,dx)
c
      mid=(nobs-1)/2+1
      call ProgName_WF('Gradcomp: Filtered Gradient Components',1)
      call ProgUpdate_WF(0)
      iprogt=mid-1+nobs-1+nr-mid-mid+1+mid-2
      iprinc = (iprogt-2)/150       
      if(iprinc.lt.2) inc=0
      
      do i=1,nc
        grad(i)=dummy
        xx(i)=dummy
        yy(i)=dummy
      enddo
      do j=1,mid-1
        if(iprinc.eq.0 .or. MOD(j,iprinc).eq.0) then
            call ProgUpdateL_WF(j-1,iprogt)
            call CheckStop_WF(ierr)
            if(ierr.eq.1) goto 9999
        endif
        call PutRow_WF(outhunit,j,nc,grad,ierr)
        if(ierr.ne.0) go to 9999
        call PutRow_WF(outxunit,j,nc,xx,ierr)
        if(ierr.ne.0) go to 9999
        call PutRow_WF(outyunit,j,nc,yy,ierr)
        if(ierr.ne.0) go to 9999
      enddo
      do j=1,nobs-1
        if(iprinc.eq.0 .or. MOD(j,iprinc).eq.0) then
            call ProgUpdateL_WF(mid-1+j-1,iprogt)
            call CheckStop_WF(ierr)
            if(ierr.eq.1) goto 9999
        endif
        call GetRow_WF(inunit,j,nc,surf(1,j),ierr)
        if(ierr.ne.0) go to 9999
      enddo
      do k=mid,nr-mid+1
        if(iprinc.eq.0 .or. MOD(k,iprinc).eq.0) then
            call ProgUpdateL_WF(mid-1+nobs-1+k-1,iprogt)
            call CheckStop_WF(ierr)
            if(ierr.eq.1) goto 9999
        endif
        call GetRow_WF(inunit,k+mid-1,nc,surf(1,nobs),ierr)
        if(ierr.ne.0) go to 9999
c
        call convolve(surf,grad,xx,yy,filtop1,filtop2,nc,ncmax,nobs)
c
        call PutRow_WF(outhunit,k,nc,grad,ierr)
        if(ierr.ne.0) go to 9999
        call PutRow_WF(outxunit,k,nc,xx,ierr)
        if(ierr.ne.0) go to 9999
        call PutRow_WF(outyunit,k,nc,yy,ierr)
        if(ierr.ne.0) go to 9999
        do j=1,nobs-1
          do i=1,nc
            surf(i,j)=surf(i,j+1)
          enddo
        enddo
      enddo
c
      do i=1,nc
        grad(i)=dummy
      enddo
      do j=nr-mid+2,nr
        if(iprinc.eq.0 .or. MOD(j,iprinc).eq.0) then
            call ProgUpdateL_WF(mid-1+nobs-1+nr-mid-mid+1+j-1,iprogt)
            call CheckStop_WF(ierr)
            if(ierr.eq.1) goto 9999
        endif
        call PutRow_WF(outhunit,j,nc,grad,ierr)
        if(ierr.ne.0) go to 9999
        call PutRow_WF(outxunit,j,nc,grad,ierr)
        if(ierr.ne.0) go to 9999
        call PutRow_WF(outyunit,j,nc,grad,ierr)
        if(ierr.ne.0) go to 9999
      enddo
9999  continue
      if(iunit.ne.0) call CloseGrid_WF(inunit,ierr)
      if(outhunit.ne.0) call CloseGrid_WF(outhunit,ierr)
      if(outxunit.ne.0) call CloseGrid_WF(outxunit,ierr)
      if(outyunit.ne.0) call CloseGrid_WF(outyunit,ierr)
      return
      end
c
      subroutine gradop(filtop1,filtop2,nobs,norder,grdint)
c
c This subroutine generates two gradient-component operators
c using the method from Thurston and Brown (Geophysics, 1994)
c Input variable:
c    nobs   - size of gradient-component operator
c Output variables:
c    filtop1 - d/dx gradient-component operator
c    filtop2 - d/dy gradient component operator
c Internal variables:
c    orpol   - array of 1d orthogonal polynomials stored as
c              a 2d array according to norder and abscissa
c    dorpol  - d/dx of orpol
c    den     - sum of squares of orthogonal polynomials
c
      parameter (maxord=20,maxnobs=21)
      real orpol(-2:maxord,maxnobs),dorpol(-2:maxord,maxnobs)
      real den(0:maxord), filt(maxnobs), dfilt(maxnobs)
      real filtop1(nobs,nobs), filtop2(nobs,nobs)
      mid=(nobs+1)/2
c calculate orthogonal polynomials and derivatives
      call orpoly(norder,nobs,orpol,dorpol)
c calculate gradient-component operators
      do i=0,norder
      do j=1,nobs
        den(i)=0.
      enddo
      enddo
c calculate denominator
      do i=0,norder
      do j=1,nobs
        den(i)=den(i)+orpol(i,j)**2
      enddo
      enddo
c calculate 1-D smoothing and derivative operators
      do 10 i=1,nobs
        filt(i)=0.
        dfilt(i)=0.
      do 11 j=0,norder
        filt(i)=filt(i)+orpol(j,i)*orpol(j,mid)/den(j)
        dfilt(i)=dfilt(i)+orpol(j,i)*dorpol(j,mid)/den(j)
   11 continue
   10 continue
c combine 1-D operators to generate 2-D operators
      do 15 i=1,nobs
      do 16 j=1,nobs
        filtop1(i,j)=filt(j)*dfilt(i)/grdint
        filtop2(i,j)=filt(i)*dfilt(j)/grdint
   16 continue
   15 continue
      return
      end
c
      subroutine orpoly(norder,nobs,orpol,dorpol)
c this subroutine generates orthogonal polynomials in x, and
c their first derivatives wrt x.
      parameter(maxord=20,maxnobs=21)
      real orpol(-2:maxord,maxnobs), dorpol(-2:maxord,maxnobs)
      real ssqpol(0:maxord),x(maxnobs)
      integer t
c initialize array containing x
      do 8 i=1,nobs
        x(i)=i
    8 continue
c initialize the first two elements of orpol and dorpol for all x
      do 1 i=1,nobs
        orpol(-2,i)=-1.
        orpol(-1,i)=0.
        dorpol(-2,i)=0.
        dorpol(-1,i)=0.
    1 continue
c generate successive polynomials using recursion
      alpha=0.0
      beta=1.0
      oldsqsum=1.0
      do 2 t=0,norder
        ssqpol(t)=0.
        xprod=0.
        do 3 i=1,nobs
          orpol(t,i)=orpol(t-2,i)*beta + orpol(t-1,i)*(x(i)+alpha)
          dorpol(t,i)=dorpol(t-2,i)*beta + dorpol(t-1,i)*(x(i)+alpha)
     +               + orpol(t-1,i)
          d=orpol(t,i)**2
          ssqpol(t)=ssqpol(t)+d
          xprod=xprod+d*x(i)
    3   continue
        beta=-ssqpol(t)/oldsqsum
        oldsqsum=ssqpol(t)
        alpha=-xprod/oldsqsum
    2 continue
      return
      end
c
      subroutine convolve(surf,grad,xx,yy,filtop1,filtop2,nc,ncmax,nobs)
      real surf(ncmax,nobs),grad(nc),xx(nc),yy(nc)
      real filtop1(nobs,nobs), filtop2(nobs,nobs)
      data dummy/-1.0e32/
c
      midc=(nobs-1)/2+1
      do 70 kc=midc,nc-midc+1
        do 41 j=1,nobs
        do 40 i=1,nobs
          if(surf(i+kc-midc,j).eq.dummy) then
            grad(kc)=dummy
            xx(kc)=dummy
            yy(kc)=dummy
            go to 70
          endif
   40   continue
   41   continue
        grad1=0.
        grad2=0.
        do 51 j=1,nobs
        do 50 i=1,nobs
          grad1=grad1+surf(i+kc-midc,j)*filtop1(i,j)
          grad2=grad2+surf(i+kc-midc,j)*filtop2(i,j)
   50   continue
   51   continue
        grad(kc)=sqrt(grad1**2+grad2**2)
        xx(kc)=grad1
        yy(kc)=grad2
   70   continue
        return
        end
